
#include "xbasic_types.h"
#include "xio.h"

#define HORZ_RES 320
#define VERT_RES 240
#define W 320
#define H 240
#define MIN_X_DIM 2
#define MIN_Y_DIM 2
#define TOLERANCE 10
#define color_match 0xC0
#define VGA_START 0x00800000

/* 
   The way this search algorithm works is quite simple. It first traverses 
   through the video_sram, looking for a particular "color_match" in a 
   sequence. From the current starting position in each line, it calculates
   the number of consecutive instances of "color_match" it can find and stores
   it to a 1-D array (each element corresponding to each horizontal line. 

   Then the second segment of the code actually traverses ONLY through this 
   1-D array created and looks for a consistent run of values that are more 
   than the "MIN_X_DIM". Then once this traversal is complete, it now knows 
   how many acceptable values of "color_run" occur in a sequence. Then it 
   stores that value temporarily along with the corresponding Y value the 
   sequence started at.

   Finally, the values of x_dim_tmp and y_dim_tmp are compared with the current
   values of x_dim and y_dim. Currently the decision is based on area, and if 
   need be to speed up the program, it can always be changed to a simple comparison
   between x_dim, y_dim and X_dim_tmp and y_dim_tmp.
*/ 

void track_object()
{
  int line, pix;
  int color_run;
  int count;
  int i, j, x; 
  int upperbound, lowerbound;
  int constraint_y;
  int x_dim, y_dim;
  int x_pos, y_pos;
  int y_pos_tmp, x_pos_tmp;
  int x_dim_tmp, y_dim_tmp;
  Xuint8 video_byte;
  int flag_x_fail = 0;

  count = 0;
  i = 0;
  j = 0;
  x_dim=0;
  y_dim=0;
  constraint_y = 0;
  upperbound = color_match + TOLERANCE;
  lowerbound = color_match - TOLERANCE;  

  for(x = 0; x < HORZ_RES; x++)
  {
    // Traversing through video_sram to get the values of 
    // matches for a particular color within a "tolerance"
       
    // The program can be altered very easily to change the
    // search color on the fly looking for any number of colors
    // by changing the color_match variable. 
    

    // setting the current x_dim_tmp //
    // to a relative infinity        //
    x_dim_tmp = HORZ_RES + 1;

    constraint_y = 0;
    y_dim_tmp = 0;
    x_pos_tmp = x; 

    for (i = 0; i < VERT_RES ; i++)
    {
      color_run = 0;
      flag_x_fail = 0;
      /* 
	 Inner loop that checks values from the current x 
	 position till the end of the video_sram array
	 
	 This is the ONLY place where video_sram is being accessed.
      */
      for(j = x; ((j < HORZ_RES)&&(flag_x_fail == 0)); j++) 
      {
        
	// Checks to see if the current videosram value is out of range
	// if the mismatch was found then the fail_x_flag is set as we 
	// are not interested in any of the values that would occur after
	// the first anomalitiy.

	video_byte = XIo_In8(VGA_START + j + 640*i);
	if((video_byte > upperbound ) || (video_byte < lowerbound ))
	{
	  flag_x_fail =1;
	  break;
	}
	else // else adds one to it's counter
	{
	  color_run++;
	}
      }

      // Here is where the 2-d recognition comes into play. This is 
      // the second part of the algorithm where it looks for the 
      // consistency in consistency of color values per se.     
      
      // If it finds a decent consistency (i.e. one that satisfies
      // both the x_dim and y_dim constraints) then it will store 
      // the x coordinate in x_pos_tmp and y coordinate in y_pos_tmp
      // and the corresponding x and y dimensions in x_dim_temp and 
      // y_dim_temp respectivley. 
      
      // We have just encountered a place where we have seen that 
      // the color_run value is more than the minimum x dimension
      // so we have to first increment the constraint_y value and
      // possibly update the y_pos_tmp value to the index of the 
      // start of the sequence. 
      // 
      // Also in the current color_run value was found to be lower
      // than the current x_dim_tmp, then you need to reassign it 
      // as the x dimension of the rectangle will be decided by the
      // lowest value of color_run in acceptable range of x_dim value

      if (color_run >= MIN_X_DIM)
      {
	constraint_y++;
	if(color_run < x_dim_tmp)
	  x_dim_tmp = color_run;
	if (constraint_y > y_dim)
	{
	  y_dim_tmp = constraint_y;
	  y_pos_tmp = i - constraint_y + 1;
	}
      }

      if (color_run < MIN_X_DIM)
      {
	if (constraint_y > y_dim)
	{
	  y_dim = constraint_y;
	  y_pos = i;
	}
	constraint_y = 0;
      }
    }
    
    
    
    // Now that the program has the values of the last best rectangle,
    // it compares it with the x and y dimensions of the one that was 
    // just found (if any). 
      
    // Currently the reassignment is made if the new rectangle has a 
    // greater area than the older one. But for optimization, this can
    // be tossed for simpler decision criteia.

    if ((x_dim_tmp * y_dim_tmp) > (x_dim * y_dim)) // Can be replaced with simple comparison
    {
      x_dim = x_dim_tmp;
      y_dim = y_dim_tmp;
      x_pos = x_pos_tmp;
      y_pos = y_pos_tmp;
    }
    
    
  } // main for loop

  
  // These are the values that are read //
  // out of the image and may be written //
  // to any part of the memory.          //
  
  for (j = x_pos-2; j < x_pos+ 2 + x_dim; j+=2)
  {
    XIo_Out8(VGA_START + j + 640*(y_pos) ,0x95);
    XIo_Out8(VGA_START + j + 640*(y_pos+y_dim) ,0x95);
  }

  for (i = y_pos-2; i < y_pos+ 2 + y_dim; i+=2)
  {
    XIo_Out8(VGA_START + x_pos + 640*(i) ,0x95);
    XIo_Out8(VGA_START + x_pos+x_dim + 640*(i) ,0x95);
  }

}

